/*
 * IBM Technical Support Appliance Version 2 Release 7 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery',
    'tsa/TypeVerificator',
    'tsa/cmd/handlers/AbstractHandler',
    'tsa/widgets/ContentArea',
    'tsa/widgets/TablePanel',
    'tsa/widgets/ButtonFactory',
    'tsa/widgets/FormPanel',
    'tsa/cmd/ShowCommand',
    'tsa/cmd/Command',
    'tsa/widgets/IconFactory'
  ],

  function( $,
            TV,
            AbstractHandler,
            ContentArea,
            TablePanel,
            ButtonFactory,
            FormPanel,
            ShowCommand,
            Command,
            IconFactory)
{
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'ScopesetshmcHandler' );

    //-------------------------------------------------------------------------

    /** 
     * @constructor
     * @extends tsa.cmd.AbstractHandler
     * Parameters added by this class (in addition to super-class' parameters):
     *   none.
     *
     * @param {Object} parameters The parameters. 
     */

    tsa.cmd.handlers.ScopesetshmcHandler = function(parameters) {
        if (!TV.typeIsObject(typeof parameters) || (parameters === null) ) {
    	    parameters = /** @type {?} */ ( {} );
        }

        // Calling the constructor of the super-class...
        AbstractHandler.prototype.constructor.call(this, parameters);
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.ScopesetshmcHandler.prototype = new AbstractHandler( {} );
    ( tsa.cmd.handlers.ScopesetshmcHandler.prototype ).constructor = tsa.cmd.handlers.ScopesetshmcHandler;

    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** MAIN SCREEN - Provide list of dynamic scope sets                       ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/
    tsa.cmd.handlers.ScopesetshmcHandler.prototype._index = function() {
    	/** @type {Object} */ var i18n = tsa.i18n;
        
        /** @type {tsa.widgets.ContentArea} */
        var contentArea = new ContentArea( { title         : i18n.labelDiscoHMCScopesHeading,
                                             helpDoc       : 'sshelp_discovery_hmc_scope_sets_index_panel.html',
                                             showBackToTop : true } );
        /** @type {tsa.widgets.TablePanel} */
        var tablePanel = new TablePanel( { title    : i18n.labelDiscoHMCScopesHeading,
                                           emptyMsg : i18n.labelDiscoHMCScopesNoSets,
                                           columns  : [ { id : 'name'   , label : i18n.labelNameColumn    , width : '80%' , sortable : false  },
                                                        { id : 'action' , label : i18n.labelActionsColumn , width : '20%' , sortable : false } ] } );

        var buttonContainer = ButtonFactory.buildButtonContainer();
        contentArea.appendMessage( { text : i18n.labelDiscoHMCScopesCaption } );

        /*
         * Build the Add button to add an HMC Dynamic Scope Set.
         */
        var addButton = ButtonFactory.buildButton( { type    : 'add',
                                                     grayout : true,
                                                     label   : i18n.buttonDiscoHMCScopesAddSet } );

        $(addButton).on( 'click', function( ) {
            (new ShowCommand( { action              : 'scopesetshmc/new',
                                clearSystemMessages : true } )
            ).execute();
        } );

        $(buttonContainer).append(addButton);

        /*
         * Run command to get all HMC Dynamic Scope Sets
         */
        var getScopesetsCmd = new Command( { action              : 'scopesets/getScopeSets',
                                             parameters          : { 'filter' : 'HMC' },
                                             clearSystemMessages : false });

        getScopesetsCmd.execute();
        $(getScopesetsCmd).on('executionSucceeded', function(event) {
            var scopesets = event.response.payload.scopeSets;
            var result    = null;
        
            if (!TV.typeIsUndefined(typeof scopesets)) {
                /*
                 * Find just the HMC scope sets.  Eliminate the hidden AIX, Linux, and VIOS scope sets when creating
                 * the list of HMC Dynamic Scope Sets.
                 */
            	var hmcScopesets = [];
            	$.each(scopesets, function(icaIdx, set) {
            		if (!set.name.endsWith("_AIX") && !set.name.endsWith("_LINUX") && !set.name.endsWith("_VIOS")) {
            			hmcScopesets.push(set);
            		}
            	} );
            	
                /*
                 * Create a row in the table for each HMC Dynamic Scope Set.
                 */
                $.each(hmcScopesets, function(icaIdx, set) {
                    var divName = document.createElement("div");
                    var aName   = document.createElement("a");
              
                    var clickDisplayHandler = function (event) {
                        (new ShowCommand( { action              : event.data.URL,
                                            clearSystemMessages : false,
                                            parameters          : { pid : event.data.pid } } )
                        ).execute();
                    };
                    
                    /**@type function() */
                    var clickTriggerHandler = function(event) {
                        tsa.clearSysMessages();
                        var proceed = true;
                        if (!TV.typeIsUndefined(typeof event.data.confirm)) {
                            proceed = confirm(tsa.bind(tsa.i18n.labelDeleteConfirmation, event.data.name ));
                        }
                        if (proceed) {
                            var actionCmd = new Command( { action              : event.data.URL,
                                                           clearSystemMessages : false } );
                            actionCmd.execute();
                            $(actionCmd).on('executionSucceeded', function() {
                                (new ShowCommand( { action              : 'scopesetshmc/index',
                                                    clearSystemMessages : false}
                                )).execute();
                            } );
                        }
                    };
                
                    $( aName ).attr( 'href', '#' );
                    $( aName ).on( 'click', { pid : set.pid, 
                	                          URL : 'scopesetshmc/show', 
                	                          pid : set.pid } , clickDisplayHandler);

                    /*
                     * Create the action icons and add to row for this dynamic scope set.
                     */
                    var dActions   = document.createElement("div");
                    var editIcon   = IconFactory.buildActionIcon( { type    : 'edit', 
                    	                                            caption : tsa.bind(i18n.labelButtonEdit , set.name )} );
                    var runHoverText = tsa.bind(i18n.labelDiscoHMCStartDiscovery, set.name);
                    var runIcon    = IconFactory.buildActionIcon( { type    : 'runb', 
                                                                    caption :  runHoverText } );
                    
                    var deleteIcon = IconFactory.buildActionIcon( { type    : 'delete', 
                	                                                caption : tsa.bind(i18n.labelButtonDelete , set.name )} );
                    var statusIcon = null;
             
                    if (set.enabled) {
                        statusIcon = IconFactory.buildActionIcon( { type    : 'active', 
                    	                                            caption : tsa.bind(i18n.labelCollectionEnabled,set.name)} );
                    } else {
                        statusIcon = IconFactory.buildActionIcon( { type    : 'offline',  
                        	                                        caption : tsa.bind(i18n.labelCollectionDisabled,set.name)} );
                    }
                
                    $( editIcon   ).on( 'click', { URL : 'scopesetshmc/edit', 
              	                                   pid : set.pid  } , clickDisplayHandler );
                    $( statusIcon ).on( 'click', { URL  : 'scopesets/ajaxToggle/'   + set.pid, 
            	                                   name : set.name   } , clickTriggerHandler );
                    $( runIcon    ).on( 'click', { URL  : 'scopesetshmc/ajaxDiscoverScopeSet/' + set.pid } , clickTriggerHandler );
                    $( deleteIcon ).on( 'click', { URL     : 'scopesetshmc/ajaxDelete/'   + set.pid, 
            	                                   name    : set.name, 
            	                                   confirm : true} , clickTriggerHandler );

                    $( divName ).append( tsa.NBSP);
                    $( divName ).append(aName);
                    $( aName   ).append( set.name);

              
                    var aRef    = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( editIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    aRef    = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( deleteIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    aRef    = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( statusIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    aRef    = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( runIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    result = [ divName , dActions ];

                    tablePanel.appendRow( result );
                } );

            }
        } );

        contentArea.append( tablePanel.getDomNode() );
        contentArea.append(buttonContainer);
        this.show( contentArea.getDomNode() );
    };
    
    
    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** NEW SCOPE SET SCREEN                                                   ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.ScopesetshmcHandler.prototype._new = function() {
    	/** @type {Object} */ var i18n = tsa.i18n;
        
        /** @type {tsa.widgets.ContentArea} */
        var contentArea = new ContentArea ( { title         : i18n.labelHMCScopeSetHeading,
                                              helpDoc       : 'sshelp_discovery_hmc_scope_set_new_panel.html' } );

        /** @type {Element} */ var buttonContainer = ButtonFactory.buildButtonContainer();
        
        /** @type {tsa.widgets.FormPanel} */ var scopeSetPanel  = new FormPanel( { title : i18n.labelDescribeScopeSetHeading } );
        /** @type {tsa.widgets.FormPanel} */ var scopePanel     = new FormPanel( { title : i18n.labelDiscoHMCScopesAddressHeading } );
        /** @type {tsa.widgets.FormPanel} */ var hmcPanel       = new FormPanel( { title       : i18n.labelDiscoHMCCredsEnterAccessHeading,
                                                                                   description : i18n.labelAccessInformationPaneDescription });
        /** @type {tsa.widgets.FormPanel} */ var lparTypePanel = new FormPanel( { title       : i18n.labelDiscoHMCCredsEnterLPARSectionHeading,
                                                                                  description : i18n.labelDiscoHMCCredsEnterLPARSectionDesc });
        /** @type {tsa.widgets.FormPanel} */ var hmcAixPanel   = new FormPanel( { title       : i18n.labelDiscoHMCCredsEnterAccessAixHeading,
                                                                                  description : i18n.labelAccessInformationPaneDescription });
        /** @type {tsa.widgets.FormPanel} */ var testFormAix   = new FormPanel( { title       : i18n.labelDiscoHMCCredsAixTestHeading,
                                                                                  description : i18n.labelDiscoHMCCredsTestDesc } );
        /** @type {tsa.widgets.FormPanel} */ var hmcLinuxPanel = new FormPanel( { title       : i18n.labelDiscoHMCCredsEnterAccessLinuxHeading,
                                                                                  description : i18n.labelAccessInformationPaneDescription });
        /** @type {tsa.widgets.FormPanel} */ var testFormLinux = new FormPanel( { title       : i18n.labelDiscoHMCCredsLinuxTestHeading,
                                                                                  description : i18n.labelDiscoHMCCredsTestDesc } );
        /** @type {tsa.widgets.FormPanel} */ var hmcViosPanel  = new FormPanel( { title       : i18n.labelDiscoHMCCredsEnterAccessViosHeading,
                                                                                  description : i18n.labelAccessInformationPaneDescription });
        /** @type {tsa.widgets.FormPanel} */ var testFormVios  = new FormPanel( { title       : i18n.labelDiscoHMCCredsViosTestHeading,
                                                                                  description : i18n.labelDiscoHMCCredsTestDesc } );
 
        // Link all FormPanel
        scopeSetPanel.linkTo(scopePanel);
        scopePanel   .linkTo(hmcPanel);
        hmcPanel     .linkTo(lparTypePanel);
        lparTypePanel.linkTo(hmcAixPanel);
        hmcAixPanel  .linkTo(testFormAix);
        testFormAix  .linkTo(hmcLinuxPanel);
        hmcLinuxPanel.linkTo(testFormLinux);
        testFormLinux.linkTo(hmcViosPanel);
        hmcViosPanel .linkTo(testFormVios);

        // Create Scope Set fields
        scopeSetPanel.setDescription( i18n.labelHMCScopeSetEnterDesc);
        scopeSetPanel.appendFieldRow( { label    : i18n.labelScopeSetNameField,
                                        required : true,
                                        element  : $( '<input size="50" maxlength="250" name="name" id="name" value="" aria-required="true" class="ibm-text" type="text">' ).get(0) } );

        scopePanel.appendFieldRow( {  label    : i18n.labelIpAddressField,
                                      required : true,
                                      element  : $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0),
                                      hint     : i18n.labelDiscoHMCScopesAddressAddressDesc } );
      
        var scopeSetGroup = $("<span class='ibm-input-group' role='group' aria-label='Scopes'></span>").get(0);
      
        // Define check box for AIX partitions.
        //
        // Create function that hides or displays the panes associated with AIX partitions as the
        // check box is selected / deselected.
        scopesetInput = $("<input name='lpars' class='checkBoxClass' type='checkbox' checked>").get(0);
        $(scopesetInput).on('click', function() {
        	console.log(this.value);          
          
        	if (this.checked) {
                hmcAixPanel.show();
                testFormAix.show();
            } else {
                hmcAixPanel.hide();
                testFormAix.hide();
            }
        });
        $(scopesetInput).attr("id", "checkBoxAix");
        $(scopesetInput).attr("value", "AIX");
        var scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
        $(scopeLabel).text("AIX");
        $(scopeLabel).attr("for", "AIX");
        $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));
      
        // Define check box for Linux partitions.
        //
        // Create function that hides or displays the panes associated with Linux partitions as the
        // check box is selected / deselected.
        scopesetInput = $("<input name='lpars' class='checkBoxClass' type='checkbox' checked>").get(0);
        $(scopesetInput).on('click', function() {
            console.log(this.value);
          
            if (this.checked) {
                hmcLinuxPanel.show();
                testFormLinux.show();
            } else {
                hmcLinuxPanel.hide();
                testFormLinux.hide();
            }
        });
        $(scopesetInput).attr("id", "checkBoxLinux");
        $(scopesetInput).attr("value", "Linux");
        scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
        $(scopeLabel).text("Linux");
        $(scopeLabel).attr("for", "Linux");
        $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));
      
        // Define check box for VIOS partitions.
        //
        // Create function that hides or displays the panes associated with VIOS partitions as the
        // check box is selected / deselected.
        scopesetInput = $("<input name='lpars' class='checkBoxClass' type='checkbox' checked>").get(0);
        $(scopesetInput).on('click',function(){
            console.log(this.value);
          
            if (this.checked) {
                hmcViosPanel.show();
                testFormVios.show();
            } else {
                hmcViosPanel.hide();
                testFormVios.hide();
            }
        });
        $(scopesetInput).attr("id", "checkBoxVios");
        $(scopesetInput).attr("value", "VIOS");
        scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
        $(scopeLabel).text("VIOS");
        $(scopeLabel).attr("for", "VIOS");
        $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));
      
        lparTypePanel.appendFieldRow( { label    : i18n.labelDiscoHMCCredsEnterLPARSectionTypes,
                                        required : false,
                                        aRequire : false,
                                        element  : scopeSetGroup } );
      
        // Create HMC Scope Set Credential fields, here we only allow all the types of credentials
        
        // Radio buttons for selecting authentication type.
        /** @type {Element} */ var hmcAuthTypePasswordRadio = document.createElement( 'input' );
        /** @type {Element} */ var hmcAuthTypePkiRadio      = document.createElement( 'input' );
        
        // Authentication type radio buttons...
        hmcAuthTypePasswordRadio.type  = 'radio';
        hmcAuthTypePasswordRadio.name  = 'HostAuth_authtype';
        hmcAuthTypePasswordRadio.value = 'authType_password';
        
        hmcAuthTypePkiRadio.type  = 'radio';
        hmcAuthTypePkiRadio.name  = 'HostAuth_authtype';
        hmcAuthTypePkiRadio.value = 'authType_pki';
        
       	// On create credential, password is always the default.
        hmcAuthTypePasswordRadio.checked = true;

        var hmcHostCredName  = $( '<input type="text"     id="HostAuth_name"             name="HostAuth_name"             size="50" maxlength="250" />' ).get(0);
        var hmcHostUserName  = $( '<input type="text"     id="HostAuth_username"         name="HostAuth_username"         size="50" maxlength="250" />' ).get(0);
        var hmcHostPassword1 = $( '<input type="password" id="HostAuth_password"         name="HostAuth_password"         size="50" maxlength="250" />' ).get(0);
        var hmcHostPassword2 = $( '<input type="password" id="HostAuth_password_confirm" name="HostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);
         
        var hmcHostUploadKey     = $( '<input type="file" id="HostAuth_sshKey" name="HostAuth_sshKey" />' ).get(0);
        var hmcHostUploadKeyType = document.createElement( 'p' ) ;

        var hmcHostUploadKeyDiv  = document.createElement( 'div' );

        $( hmcHostUploadKeyType ).text( i18n.labelDiscoHMCCredsInternalSSHKey );

        $( hmcHostUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
        $( hmcHostUploadKeyDiv ).append( hmcHostUploadKeyType );
        $( hmcHostUploadKeyDiv ).append( hmcHostUploadKey     );
        
        /*
         * COMMON EVENT HANDLER - Notified when the "Authentication type" radio buttons are changed for HMC, AIX, Linux, or VIOS.
         */
        var onHostAuthAuthTypeChanged = function( event ) {
    	    // Elements
            var panel          = event.data.hmcPanel;
            var uploadKey      = event.data.hmcHostUploadKey;
            var uploadKeyType  = event.data.hmcHostUploadKeyType;
            var passwordRadio  = event.data.hmcAuthTypePasswordRadio;
            var pkiRadio       = event.data.hmcAuthTypePkiRadio;
            var hostPassword1  = event.data.hmcHostPassword1;
            var hostPassword2  = event.data.hmcHostPassword2;
            var row            = event.data.rowPanel;
            var uploadKeyField = uploadKey ;
            var keyTypeMessage = uploadKeyType;
          
            // String
            var authSshKey     = event.data.hostAuthSshKey;
            
        	/** @type {boolean} */ var isPKI = false;
        	if (pkiRadio.checked) {
        		isPKI = true;
        	}

        	// 
        	// All checks are really based on whether this is a password or PKI credential, but for code clarity
        	// purposes, create some variables that indicate what portion of the credential page should be shown
        	// or hidden.
        	//
        	/** @type {boolean} */ var hideSSHKeyRow       = !isPKI;
        	/** @type {boolean} */ var hideSSHKeyUpload    = hideSSHKeyRow;
        	/** @type {boolean} */ var disableSSHKeyUpload = !isPKI;
        	/** @type {boolean} */ var hideKeyTypeMessage  = isPKI ;

            $( row            )[ hideSSHKeyRow      ? 'hide' : 'show' ]();
            $( uploadKeyField )[ hideSSHKeyUpload   ? 'hide' : 'show' ]();
            $( keyTypeMessage )[ hideKeyTypeMessage ? 'hide' : 'show' ]();
          
            if (hideSSHKeyUpload){
                panel.removeValidation( authSshKey );
            } else {
                panel.addValidationRule( authSshKey, 'required' );
            }
            // Remove required field for passphrase
            if (isPKI) {
                panel.removeValidation( $(hostPassword1).attr('id') );
                panel.removeValidation( $(hostPassword2).attr('id') );
            } else {
                panel.addValidationRule( $(hostPassword1).attr('id'), 'required' );
                panel.addValidationRule( $(hostPassword2).attr('id'), 'required' );
            }
          
            // Password or Passphare? use the right label...
            var labelValue = isPKI ? i18n.labelPassphraseField : i18n.labelPasswordField ;
            var labelNode  = $( hostPassword1 ).closest( 'tr' ).find( 'label' );
            labelNode.empty();
          
            if (!isPKI) {
                labelNode.append(labelValue, $("<span class='ibm-required'> *</span>").get(0));
            } else {
                labelNode.append(labelValue);
            }
            labelValue = isPKI ? i18n.labelConfirmPassphraseField : i18n.labelConfirmPasswordField ;
            labelNode = $( hostPassword2 ).closest( 'tr' ).find( 'label' );
            labelNode.empty();
            if (!isPKI) {
                labelNode.append(labelValue, $("<span class='ibm-required'> *</span>").get(0));
            } else {
                labelNode.append(labelValue);
            }

            if (isPKI) {
                keyTypeMessage = i18n.labelDiscoHMCCredsExternalSSHKey;
            }
        }

        hmcPanel.appendFieldRow( { label: i18n.labelCredentialName , required: true  , element: hmcHostCredName , smallLabel : tsa.isUserLanguageCode(tsa.GERMAN_LANG_CODE)  } );
        
        hmcPanel.appendFieldRow( {
            label    : i18n.labelAuthenticationTypeField
          , required : true
          , aRequire : false
          , element  : ( function()
                         {
                           /** @type {Element} */ var span          = document.createElement( 'span' );
                           /** @type {Element} */ var passwordLabel = document.createElement( 'label' );
                           /** @type {Element} */ var pkiLabel      = document.createElement( 'label' );
                           
                           $( passwordLabel ).append( hmcAuthTypePasswordRadio, document.createTextNode( i18n.labelCredentialsAuthenticationTypePassword  ) );
                           $( pkiLabel      ).append( hmcAuthTypePkiRadio,      document.createTextNode( i18n.labelCredentialsAuthenticationTypePkiScoped ) );
                           $( span          ).append( passwordLabel, document.createElement( 'br' ), pkiLabel );
                           $( span          ).addClass( 'ibm-input-group' );
                           $( span          ).attr( 'role', 'radiogroup' );
                           $( span          ).attr( 'aria-label', i18n.scheduleDayMode );
                           
                           return span;
                         } )()
        } );
        
        hmcPanel.appendFieldRow( { label: i18n.labelUserNameField        , required: true , element: hmcHostUserName  } );
        hmcPanel.appendFieldRow( { label: i18n.labelPasswordField        , required: true , element: hmcHostPassword1 } );
        hmcPanel.appendFieldRow( { label: i18n.labelConfirmPasswordField , required: true , element: hmcHostPassword2 } );
      
        var rowPanel = hmcPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsSSHKey , required: true , element: hmcHostUploadKeyDiv } );
        hmcPanel.removeValidation( "HostAuth_sshKey" );
      
        //
        // Event handler for HMC credential if user changes the authentication type pull-down value.
        //
        $( hmcAuthTypePasswordRadio ).add( hmcAuthTypePkiRadio ).on( 'change',
            { hmcPanel                 : hmcPanel,
              hmcHostUploadKey         : hmcHostUploadKey,
              hmcHostUploadKeyType     : hmcHostUploadKeyType,
              hmcAuthTypePasswordRadio : hmcAuthTypePasswordRadio,
              hmcAuthTypePkiRadio      : hmcAuthTypePkiRadio,
              hmcHostPassword1         : hmcHostPassword1,
              hmcHostPassword2         : hmcHostPassword2,
              hmcHostUploadKeyDiv      : hmcHostUploadKeyDiv,
              rowPanel                 : rowPanel,
              hostAuthSshKey           : "HostAuth_sshKey"
            },
            onHostAuthAuthTypeChanged );
      
        // Values to enable default configuration
        var HMCAuthValues  = new Object();
        HMCAuthValues.data = new Object();
        HMCAuthValues.data.hmcPanel                 = hmcPanel;
        HMCAuthValues.data.hmcHostUploadKey         = hmcHostUploadKey;
        HMCAuthValues.data.hmcHostUploadKeyType     = hmcHostUploadKeyType;
        HMCAuthValues.data.hmcAuthTypePasswordRadio = hmcAuthTypePasswordRadio;
        HMCAuthValues.data.hmcAuthTypePkiRadio      = hmcAuthTypePkiRadio;
        HMCAuthValues.data.hmcHostPassword1         = hmcHostPassword1;
        HMCAuthValues.data.hmcHostPassword2         = hmcHostPassword2;
        HMCAuthValues.data.hmcHostUploadKeyDiv      = hmcHostUploadKeyDiv;
        HMCAuthValues.data.rowPanel                 = rowPanel;
      
        // Create test credential button for HMC and define the action taken when the button is clicked.
        var testHMCButton = this.createTestButton(scopeSetPanel, 'scopesetshmc/ajaxTestHmcCredential');

        var row = hmcPanel.layoutTableBody.insertRow(-1);
        var td = document.createElement("td");
        var pButton = document.createElement("p");
        $(pButton).append(testHMCButton);
        $(td).attr("colspan", 2);
        $(td).append(pButton);
        $(row).append(td);
      
        /*
         * Create HMC AIX Scope Set Credential fields
         */ 

        // Radio buttons for selecting authentication type.
        /** @type {Element} */ var aixAuthTypePasswordRadio = document.createElement( 'input' );
        /** @type {Element} */ var aixAuthTypePkiRadio      = document.createElement( 'input' );
        
        // Authentication type radio buttons...
        aixAuthTypePasswordRadio.type  = 'radio';
        aixAuthTypePasswordRadio.name  = 'HostAuthAix_authtype';
        aixAuthTypePasswordRadio.value = 'authType_password';
        
        aixAuthTypePkiRadio.type  = 'radio';
        aixAuthTypePkiRadio.name  = 'HostAuthAix_authtype';
        aixAuthTypePkiRadio.value = 'authType_pki';
        
       	// On create credential, password is always the default.
        aixAuthTypePasswordRadio.checked = true;
        
        var aixHMCHostCredName  = $( '<input type="text"     id="HostAuthAix_name"             name="HostAuthAix_name"             size="50" maxlength="250" />' ).get(0);
        var aixHMCHostUserName  = $( '<input type="text"     id="HostAuthAix_username"         name="HostAuthAix_username"         size="50" maxlength="250" />' ).get(0);
        var aixHMCHostPassword1 = $( '<input type="password" id="HostAuthAix_password"         name="HostAuthAix_password"         size="50" maxlength="250" />' ).get(0);
        var aixHMCHostPassword2 = $( '<input type="password" id="HostAuthAix_password_confirm" name="HostAuthAix_password_confirm" size="50" maxlength="250" />' ).get(0);
 
      
        var aixHMCHostUploadKey     = $( '<input type="file" id="HostAuthAix_sshKey" name="HostAuthAix_sshKey" />' ).get(0);
        var aixHMCHostUploadKeyType = document.createElement( 'p' );

        var aixHMCHostUploadKeyDiv  = document.createElement( 'div' );
      
        $( aixHMCHostUploadKeyType ).text( i18n.labelDiscoHMCCredsInternalSSHKey );

        $( aixHMCHostUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
        $( aixHMCHostUploadKeyDiv ).append( aixHMCHostUploadKeyType );
        $( aixHMCHostUploadKeyDiv ).append( aixHMCHostUploadKey     );
      
        hmcAixPanel.appendFieldRow( { label: i18n.labelCredentialName , required : true  , element : aixHMCHostCredName , smallLabel : tsa.isUserLanguageCode(tsa.GERMAN_LANG_CODE) } );
        
        hmcAixPanel.appendFieldRow( {
            label    : i18n.labelAuthenticationTypeField
          , required : true
          , aRequire : false
          , element  : ( function()
                         {
                           /** @type {Element} */ var span          = document.createElement( 'span' );
                           /** @type {Element} */ var passwordLabel = document.createElement( 'label' );
                           /** @type {Element} */ var pkiLabel      = document.createElement( 'label' );
                           
                           $( passwordLabel ).append( aixAuthTypePasswordRadio, document.createTextNode( i18n.labelCredentialsAuthenticationTypePassword  ) );
                           $( pkiLabel      ).append( aixAuthTypePkiRadio,      document.createTextNode( i18n.labelCredentialsAuthenticationTypePkiScoped ) );
                           $( span          ).append( passwordLabel, document.createElement( 'br' ), pkiLabel );
                           $( span          ).addClass( 'ibm-input-group' );
                           $( span          ).attr( 'role', 'radiogroup' );
                           $( span          ).attr( 'aria-label', i18n.scheduleDayMode );
                           
                           return span;
                         } )()
        } );

        hmcAixPanel.appendFieldRow( { label: i18n.labelUserNameField        , required : true , element : aixHMCHostUserName     } );
        hmcAixPanel.appendFieldRow( { label: i18n.labelPasswordField        , required : true , element : aixHMCHostPassword1    } );
        hmcAixPanel.appendFieldRow( { label: i18n.labelConfirmPasswordField , required : true , element : aixHMCHostPassword2    } );
      
        var rowAixPanel = hmcAixPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsSSHKey , required: true  , element: aixHMCHostUploadKeyDiv } );
        hmcAixPanel.removeValidation( "HostAuthAix_sshKey" );
        //
        // Event handler for AIX credential if user changes the authentication type pull-down value.
        //
        $( aixAuthTypePasswordRadio ).add( aixAuthTypePkiRadio ).on('change',
            { hmcPanel                 : hmcAixPanel,
              hmcHostUploadKey         : aixHMCHostUploadKey,
              hmcHostUploadKeyType     : aixHMCHostUploadKeyType,
              hmcAuthTypePasswordRadio : aixAuthTypePasswordRadio,
              hmcAuthTypePkiRadio      : aixAuthTypePkiRadio,
              hmcHostPassword1         : aixHMCHostPassword1,
              hmcHostPassword2         : aixHMCHostPassword2,
              hmcHostUploadKeyDiv      : aixHMCHostUploadKeyDiv,
              rowPanel                 : rowAixPanel,
              hostAuthSshKey           : "HostAuthAix_sshKey"
            }
            , onHostAuthAuthTypeChanged );
       
        // Values to enable default configuration
        var AixAuthValues  = new Object();
        AixAuthValues.data = new Object();
        AixAuthValues.data.hmcPanel                 = hmcAixPanel;
        AixAuthValues.data.hmcHostUploadKey         = aixHMCHostUploadKey;
        AixAuthValues.data.hmcHostUploadKeyType     = aixHMCHostUploadKeyType;
        AixAuthValues.data.hmcAuthTypePasswordRadio = aixAuthTypePasswordRadio;
        AixAuthValues.data.hmcAuthTypePkiRadio      = aixAuthTypePkiRadio;
        AixAuthValues.data.hmcHostPassword1         = aixHMCHostPassword1;
        AixAuthValues.data.hmcHostPassword2         = aixHMCHostPassword2;
        AixAuthValues.data.hmcHostUploadKeyDiv      = aixHMCHostUploadKeyDiv;
        AixAuthValues.data.rowPanel                 = rowAixPanel;
        
        // Create test credential button for AIX and define the action taken when the button is clicked.
        var testAIXButton = this.createTestButton(scopeSetPanel, 'scopesetshmc/ajaxTestAixCredential');

        // Test form for AIX
        var testhostFieldAix = $("<input size='50' maxlength='250' name='aaddraix' id='aaddraix' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0);
        testFormAix.appendFieldRow( { label    : i18n.labelIpAddressField,   
                                      required : false, 
                                      element  : testhostFieldAix } );
        testFormAix.appendHR();
        testFormAix.appendJointRow(testAIXButton);
        
        /* 
         * Create HMC Linux Scope Set Credential fields 
         */

        // Radio buttons for selecting authentication type.
        /** @type {Element} */ var linuxAuthTypePasswordRadio = document.createElement( 'input' );
        /** @type {Element} */ var linuxAuthTypePkiRadio      = document.createElement( 'input' );
        
        // Authentication type radio buttons...
        linuxAuthTypePasswordRadio.type  = 'radio';
        linuxAuthTypePasswordRadio.name  = 'HostAuthLinux_authtype';
        linuxAuthTypePasswordRadio.value = 'authType_password';
        
        linuxAuthTypePkiRadio.type  = 'radio';
        linuxAuthTypePkiRadio.name  = 'HostAuthLinux_authtype';
        linuxAuthTypePkiRadio.value = 'authType_pki';
        
       	// On create credential, password is always the default.
        linuxAuthTypePasswordRadio.checked = true;
        
        var linuxHmcHostCredName      = $( '<input type="text"     id="HostAuthLinux_name"             name="HostAuthLinux_name"             size="50" maxlength="250" />' ).get(0);
        var linuxHmcHostUserName      = $( '<input type="text"     id="HostAuthLinux_username"         name="HostAuthLinux_username"         size="50" maxlength="250" />' ).get(0);
        var linuxHmcHostPassword1     = $( '<input type="password" id="HostAuthLinux_password"         name="HostAuthLinux_password"         size="50" maxlength="250" />' ).get(0);
        var linuxHmcHostPassword2     = $( '<input type="password" id="HostAuthLinux_password_confirm" name="HostAuthLinux_password_confirm" size="50" maxlength="250" />' ).get(0);
        
        var linuxHmcHostUploadKey     = $( '<input type="file" id="HostAuthLinux_sshKey" name="HostAuthLinux_sshKey" />' ).get(0);
        var linuxHmcHostUploadKeyType = document.createElement('p');
      
        var linuxHmcHostUploadKeyDiv  = document.createElement('div');
      
        $( linuxHmcHostUploadKeyType ).text( i18n.labelDiscoHMCCredsInternalSSHKey );
      
        $( linuxHmcHostUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
        $( linuxHmcHostUploadKeyDiv ).append( linuxHmcHostUploadKeyType );
        $( linuxHmcHostUploadKeyDiv ).append( linuxHmcHostUploadKey     );
      
        hmcLinuxPanel.appendFieldRow( { label: i18n.labelCredentialName , required : true , element : linuxHmcHostCredName , smallLabel : tsa.isUserLanguageCode(tsa.GERMAN_LANG_CODE) } );
        
        hmcLinuxPanel.appendFieldRow( {
            label    : i18n.labelAuthenticationTypeField
          , required : true
          , aRequire : false
          , element  : ( function()
                         {
                           /** @type {Element} */ var span          = document.createElement( 'span' );
                           /** @type {Element} */ var passwordLabel = document.createElement( 'label' );
                           /** @type {Element} */ var pkiLabel      = document.createElement( 'label' );
                           
                           $( passwordLabel ).append( linuxAuthTypePasswordRadio, document.createTextNode( i18n.labelCredentialsAuthenticationTypePassword  ) );
                           $( pkiLabel      ).append( linuxAuthTypePkiRadio,      document.createTextNode( i18n.labelCredentialsAuthenticationTypePkiScoped ) );
                           $( span          ).append( passwordLabel, document.createElement( 'br' ), pkiLabel );
                           $( span          ).addClass( 'ibm-input-group' );
                           $( span          ).attr( 'role', 'radiogroup' );
                           $( span          ).attr( 'aria-label', i18n.scheduleDayMode );
                           
                           return span;
                         } )()
        } );

        hmcLinuxPanel.appendFieldRow( { label: i18n.labelUserNameField                , required : true , element : linuxHmcHostUserName     } );
        hmcLinuxPanel.appendFieldRow( { label: i18n.labelPasswordField                , required : true , element : linuxHmcHostPassword1    } );
        hmcLinuxPanel.appendFieldRow( { label: i18n.labelConfirmPasswordField         , required : true , element : linuxHmcHostPassword2    } );
      
        var rowLinuxPanel = hmcLinuxPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsSSHKey   , required: true  , element: linuxHmcHostUploadKeyDiv } );
        hmcLinuxPanel.removeValidation( "HostAuthLinux_sshKey" );
        //
        // Event handler for Linux credential if user changes the authentication type pull-down value.
        //
        $( linuxAuthTypePasswordRadio ).add( linuxAuthTypePkiRadio ).on('change',
            { hmcPanel                 : hmcLinuxPanel,
              hmcHostUploadKey         : linuxHmcHostUploadKey,
              hmcHostUploadKeyType     : linuxHmcHostUploadKeyType,
              hmcAuthTypePasswordRadio : linuxAuthTypePasswordRadio,
              hmcAuthTypePkiRadio      : linuxAuthTypePkiRadio,
              hmcHostPassword1         : linuxHmcHostPassword1,
              hmcHostPassword2         : linuxHmcHostPassword2,
              hmcHostUploadKeyDiv      : linuxHmcHostUploadKeyDiv,
              rowPanel                 : rowLinuxPanel,
              hostAuthSshKey           : "HostAuthLinux_sshKey"
            }
            , onHostAuthAuthTypeChanged );
     
        // Values to enable default configuration
        var linuxAuthValues  = new Object();
        linuxAuthValues.data = new Object();
        linuxAuthValues.data.hmcPanel                 = hmcLinuxPanel;
        linuxAuthValues.data.hmcHostUploadKey         = linuxHmcHostUploadKey;
        linuxAuthValues.data.hmcHostUploadKeyType     = linuxHmcHostUploadKeyType;
        linuxAuthValues.data.hmcAuthTypePasswordRadio = linuxAuthTypePasswordRadio;
        linuxAuthValues.data.hmcAuthTypePkiRadio      = linuxAuthTypePkiRadio;
        linuxAuthValues.data.hmcHostPassword1         = linuxHmcHostPassword1;
        linuxAuthValues.data.hmcHostPassword2         = linuxHmcHostPassword2;
        linuxAuthValues.data.hmcHostUploadKeyDiv      = linuxHmcHostUploadKeyDiv;
        linuxAuthValues.data.rowPanel                 = rowLinuxPanel;
      
        // Create test credential button for Linux and define the action taken when the button is clicked.
        var testLinuxButton = this.createTestButton(scopeSetPanel, 'scopesetshmc/ajaxTestLinuxCredential');      

        // Test form for Linux
        var testhostFieldLinux = $("<input size='50' maxlength='250' name='aaddrlinux' id='aaddrlinux' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0)
        testFormLinux.appendFieldRow( { label    : i18n.labelIpAddressField,   
                                        required : false, 
                                        element  : testhostFieldLinux } );
        testFormLinux.appendHR();
        testFormLinux.appendJointRow(testLinuxButton);
      
        /*   
         * Create HMC VIOS Scope Set Credential fields
         */

        // Radio buttons for selecting authentication type.
        /** @type {Element} */ var viosAuthTypePasswordRadio = document.createElement( 'input' );
        /** @type {Element} */ var viosAuthTypePkiRadio      = document.createElement( 'input' );
        
        // Authentication type radio buttons...
        viosAuthTypePasswordRadio.type  = 'radio';
        viosAuthTypePasswordRadio.name  = 'HostAuthVios_authtype';
        viosAuthTypePasswordRadio.value = 'authType_password';
        
        viosAuthTypePkiRadio.type  = 'radio';
        viosAuthTypePkiRadio.name  = 'HostAuthVios_authtype';
        viosAuthTypePkiRadio.value = 'authType_pki';
        
       	// On create credential, password is always the default.
        viosAuthTypePasswordRadio.checked = true;

        var viosHmcHostName      = $( '<input type="text"     id="HostAuthVios_name"             name="HostAuthVios_name"             size="50" maxlength="250" />' ).get(0);
        var viosHmcHostUserName  = $( '<input type="text"     id="HostAuthVios_username"         name="HostAuthVios_username"         size="50" maxlength="250" />' ).get(0);
        var viosHmcHostPassword1 = $( '<input type="password" id="HostAuthVios_password"         name="HostAuthVios_password"         size="50" maxlength="250" />' ).get(0);
        var viosHmcHostPassword2 = $( '<input type="password" id="HostAuthVios_password_confirm" name="HostAuthVios_password_confirm" size="50" maxlength="250" />' ).get(0);
        
        var viosHmcHostUploadKey     = $( '<input type="file" id="HostAuthVios_sshKey" name="HostAuthVios_sshKey" />' ).get(0);
        var viosHmcHostUploadKeyType = document.createElement('p');
      
        var viosHmcHostUploadKeyDiv  = document.createElement('div');
      
        $( viosHmcHostUploadKeyType ).text( i18n.labelDiscoHMCCredsInternalSSHKey );
      
        $( viosHmcHostUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
        $( viosHmcHostUploadKeyDiv ).append( viosHmcHostUploadKeyType );
        $( viosHmcHostUploadKeyDiv ).append( viosHmcHostUploadKey     );

        hmcViosPanel.appendFieldRow( { label: i18n.labelCredentialName , required : true  , element : viosHmcHostName , smallLabel : tsa.isUserLanguageCode(tsa.GERMAN_LANG_CODE) } );
        
        hmcViosPanel.appendFieldRow( {
            label    : i18n.labelAuthenticationTypeField
          , required : true
          , aRequire : false
          , element  : ( function()
                         {
                           /** @type {Element} */ var span          = document.createElement( 'span' );
                           /** @type {Element} */ var passwordLabel = document.createElement( 'label' );
                           /** @type {Element} */ var pkiLabel      = document.createElement( 'label' );
                           
                           $( passwordLabel ).append( viosAuthTypePasswordRadio, document.createTextNode( i18n.labelCredentialsAuthenticationTypePassword  ) );
                           $( pkiLabel      ).append( viosAuthTypePkiRadio,      document.createTextNode( i18n.labelCredentialsAuthenticationTypePkiScoped ) );
                           $( span          ).append( passwordLabel, document.createElement( 'br' ), pkiLabel );
                           $( span          ).addClass( 'ibm-input-group' );
                           $( span          ).attr( 'role', 'radiogroup' );
                           $( span          ).attr( 'aria-label', i18n.scheduleDayMode );
                           
                           return span;
                         } )()
        } );

        hmcViosPanel.appendFieldRow( { label: i18n.labelUserNameField                 , required : true  , element : viosHmcHostUserName     } );
        hmcViosPanel.appendFieldRow( { label: i18n.labelPasswordField                 , required : true  , element : viosHmcHostPassword1    } );
        hmcViosPanel.appendFieldRow( { label: i18n.labelConfirmPasswordField          , required : true  , element : viosHmcHostPassword2    } );      
        
        var rowViosPanel = hmcViosPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsSSHKey   , required: true  , element: viosHmcHostUploadKeyDiv } );
        hmcViosPanel.removeValidation( "HostAuthVios_sshKey" );
        //
        // Event handler forVIOS credential if user changes the authentication type pull-down value.
        //
        $( viosAuthTypePasswordRadio ).add( viosAuthTypePkiRadio ).on('change',
            {  hmcPanel                 : hmcViosPanel,
               hmcHostUploadKey         : viosHmcHostUploadKey,
               hmcHostUploadKeyType     : viosHmcHostUploadKeyType,
               hmcAuthTypePasswordRadio : viosAuthTypePasswordRadio,
               hmcAuthTypePkiRadio      : viosAuthTypePkiRadio,
               hmcHostPassword1         : viosHmcHostPassword1,
               hmcHostPassword2         : viosHmcHostPassword2,
               hmcHostUploadKeyDiv      : viosHmcHostUploadKeyDiv,
               rowPanel                 : rowViosPanel,      
               hostAuthSshKey           : "HostAuthVios_sshKey"
            }, onHostAuthAuthTypeChanged );
        
        // Default values to trigger change handler
        var viosAuthValues  = new Object();
        viosAuthValues.data = new Object();
        viosAuthValues.data.hmcPanel                 = hmcViosPanel;
        viosAuthValues.data.hmcHostUploadKey         = viosHmcHostUploadKey;
        viosAuthValues.data.hmcHostUploadKeyType     = viosHmcHostUploadKeyType;
        viosAuthValues.data.hmcAuthTypePasswordRadio = viosAuthTypePasswordRadio;
        viosAuthValues.data.hmcAuthTypePkiRadio      = viosAuthTypePkiRadio;
        viosAuthValues.data.hmcHostPassword1         = viosHmcHostPassword1;
        viosAuthValues.data.hmcHostPassword2         = viosHmcHostPassword2;
        viosAuthValues.data.hmcHostUploadKeyDiv      = viosHmcHostUploadKeyDiv;
        viosAuthValues.data.rowPanel                 = rowViosPanel;

        // Create test credential button for VIOS and define the action taken when the button is clicked.
        var testVIOSButton = this.createTestButton(scopeSetPanel, 'scopesetshmc/ajaxTestViosCredential');
      
        var testhostFieldVios = $("<input size='50' maxlength='250' name='aaddrvios' id='aaddrvios' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0)
        testFormVios.appendFieldRow( { label    : i18n.labelIpAddressField,   
                                       required : false,
                                       element  : testhostFieldVios } );
        testFormVios.appendHR();
        testFormVios.appendJointRow(testVIOSButton);
      

        /*
         * Create save button and define action when button is clicked.
         */
        var saveButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      grayout : false,
                                                      label   : i18n.buttonSave } );

        $(saveButton).on('click', function() {
            scopeSetPanel.setActionAttribute("scopesetshmc/ajaxCreate/");
            scopeSetPanel.submit();

            $(scopeSetPanel).on('executionSucceeded', function(event) {
                (new ShowCommand( { action              : 'scopesetshmc/index',
                                    clearSystemMessages : false,
                                    parameters          : { pid : event.response.payload.pid} } )
                ).execute();
            } );

            $(scopeSetPanel).on('executionFailed', function(event) {
                if (TV.typeIsObject( typeof event.response.payload.errors)) {
                    for (var fieldName in event.response.payload.errors ) {
                        scopeSetPanel.showFieldError(fieldName, event.response.payload.errors[fieldName]);
                    }
                    delete event.response.payload.errors;
                }
            } );
        } );
      
        // Call onHostAuthAuthTypeChanged to trigger default values.
        onHostAuthAuthTypeChanged(HMCAuthValues);
        onHostAuthAuthTypeChanged(AixAuthValues);
        onHostAuthAuthTypeChanged(linuxAuthValues);
        onHostAuthAuthTypeChanged(viosAuthValues);


        /*
         * Create cancel button and define action when button is clicked.
         */
        var cancelButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                        grayout : true,
                                                        label   : i18n.buttonCancel } );

        $(cancelButton).on('click', function() {
            (new ShowCommand( { action              : 'scopesetshmc/index',
                                clearSystemMessages : false }
            )).execute();
        } );
        
        $(buttonContainer).append(saveButton);
        $(buttonContainer).append(ButtonFactory.buildButtonSeparator);
        $(buttonContainer).append(cancelButton);
  
        contentArea.append(scopeSetPanel.getDomNode());
        contentArea.append(scopePanel   .getDomNode());
        contentArea.append(hmcPanel     .getDomNode());
        contentArea.append(lparTypePanel.getDomNode());
        contentArea.append(hmcAixPanel  .getDomNode());
        contentArea.append(hmcLinuxPanel.getDomNode());
        contentArea.append(hmcViosPanel .getDomNode());
     
        contentArea.append( buttonContainer );

        this.show(contentArea.getDomNode());
    };


    /*
     * Set-up test credential button based on the type of credential being tested (HMC, AIX, Linux, VIOS).
     *
     * @param firstForm Form that will contain the test credential button.
     * @param actionAttribute Action taken when test credential button is clicked.
     *
     * @return Test credential button.
     */
    tsa.cmd.handlers.ScopesetshmcHandler.prototype.createTestButton = function(scopeSetPanel, actionAttribute) {
        var i18n = tsa.i18n;
        var testButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      label   : i18n.buttonTestCredential } );

        $(testButton).on('click', function( ) {
            tsa.clearSysMessages();

            var action = scopeSetPanel.getActionAttribute();  // Save current value and restore after panel is submitted.

            scopeSetPanel.setActionAttribute(actionAttribute);
            scopeSetPanel.submit();
            scopeSetPanel.setActionAttribute(action);

            $(scopeSetPanel).on('executionFailed', function(event) {
                if (TV.typeIsObject(typeof event.response.payload.errors))  {
                    for (var fieldName in event.response.payload.errors) {
                        scopeSetPanel.showFieldError(fieldName, event.response.payload.errors[fieldName]);
                    }
                    delete event.response.payload.errors;
                }
            } );
        } );

        return testButton;
    };
    

    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** SHOW SCOPE SET SCREEN                                                  ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.ScopesetshmcHandler.prototype._show = function() {
        // Redirect to edit
        this._edit();
    };
    
    
    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** EDIT SCOPE SET SCREEN                                                  ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.ScopesetshmcHandler.prototype._edit = function() {
    	/** @type {Object} */ var i18n = tsa.i18n;
    	
        var pid = this.getParameter('pid');
        // "name" is filled in by getScopesetCmd and used later by getCredCommand.  Therefore, edit has to ensure
        // getScopesetCmd completes before getCredCommand is started.
        var name = null; 
        
        /** @type {tsa.widgets.ContentArea} */
        var contentArea = new ContentArea ( { title         : i18n.labelHMCScopeSetHeading,
                                              helpDoc       : 'sshelp_discovery_hmc_scope_set_show_edit_panel.html' } );
        
        /** @type {tsa.cmd.Command} */
        var getScopesetCmd = new Command( { action              : 'scopesets/getScopeSet',
                                            clearSystemMessages : false,
                                            parameters          : { name : pid } });
  
        
        /*-----------------*/
        /* GENERAL PANE    */
        /*-----------------*/
        /** @type {tsa.widgets.TablePanel} */
        var tablePanelGeneral = new TablePanel( { title   : i18n.labelGeneralPaneHeading,
                                                  columns : [ { id : 'name'   , width : '80'}, { id : 'value' } ] } );

        /*-----------------*/
        /* HMC SCOPES PANE */
        /*-----------------*/
        /** NOTE: emptyMsg should never occur for this panel. */
        /** @type {tsa.widgets.TablePanel} */
        var tablePanelHmcScopes = new TablePanel( { title    : i18n.labelHMCScopesHeading,
                                                    emptyMsg : i18n.labelHMCScopeSetNoElements,
                                                    columns  : [  { id : 'type'     , label : i18n.labelTypeColumn    , width : '20%'},
                                                                  { id : 'value'    , label : i18n.labelValueColumn   , width : '30%'},
                                                                  { id : 'actions'  , label : i18n.labelActionsColumn , width : '15%'}
                                                               ] });

        /** @type {Element} */ var buttonContainerHmcScopes = ButtonFactory.buildButtonContainer();

        // FUTURE - create the label
        var buttonNewScope = null;

        /*------------------------*/
        /* CREDENTIALS PANE - HMC */
        /*------------------------*/
        /** NOTE: emptyMsg should never occur for this panel. */
        /** @type {tsa.widgets.TablePanel} */
        var tablePanelHmcCred = new TablePanel( { title    : i18n.labelDiscoHMCCredsHmcCredentialsHeading,
                                                  emptyMsg : i18n.labelDiscoHMCCredsNoCreds,
                                                  columns  : [  { id : 'name'          , label : i18n.labelNameColumn     , width : '20%' , sortable : false },
                                                                { id : 'type'          , label : i18n.labelTypeColumn     , width : '20%' , sortable : false },
                                                                { id : 'userName'      , label : i18n.labelUserNameColumn , width : '15%' , sortable : false },
                                                                { id : 'actions'       , label : i18n.labelActionsColumn  , width : '15%' , sortable : false } ] } );

        /** @type {Element} */ var buttonContainerHmcCredentials = ButtonFactory.buildButtonContainer();

        /*------------------------*/
        /* CREDENTIALS PANE - AIX */
        /*------------------------*/
        /** @type {tsa.widgets.TablePanel} */
        var tablePanelAixCred = new TablePanel( { title    : i18n.labelDiscoHMCCredsAixCredentialsHeading,
                                                  emptyMsg : i18n.labelDiscoHMCCredsNoCreds,
                                                  columns  : [  { id : 'name'          , label : i18n.labelNameColumn     , width : '20%' , sortable : false },
                                                                { id : 'type'          , label : i18n.labelTypeColumn     , width : '20%' , sortable : false },
                                                                { id : 'userName'      , label : i18n.labelUserNameColumn , width : '15%' , sortable : false },
                                                                { id : 'actions'       , label : i18n.labelActionsColumn  , width : '15%' , sortable : false } ] } );

        /** @type {Element} */ var buttonContainerAixCredentials = ButtonFactory.buildButtonContainer();


        /*--------------------------*/
        /* CREDENTIALS PANE - Linux */
        /*--------------------------*/
        /** @type {tsa.widgets.TablePanel} */
        var tablePanelLinuxCred = new TablePanel( { title    : i18n.labelDiscoHMCCredsLinuxCredentialsHeading,
                                                    emptyMsg : i18n.labelDiscoHMCCredsNoCreds,
                                                    columns  : [  { id : 'name'          , label : i18n.labelNameColumn     , width : '20%' , sortable : false },
                                                                  { id : 'type'          , label : i18n.labelTypeColumn     , width : '20%' , sortable : false },
                                                                  { id : 'userName'      , label : i18n.labelUserNameColumn , width : '15%' , sortable : false },
                                                                  { id : 'actions'       , label : i18n.labelActionsColumn  , width : '15%' , sortable : false } ] } );

        /** @type {Element} */ var buttonContainerLinuxCredentials = ButtonFactory.buildButtonContainer();

        /*-------------------------*/
        /* CREDENTIALS PANE - VIOS */
        /*-------------------------*/
        /** @type {tsa.widgets.TablePanel} */
        var tablePanelViosCred = new TablePanel( { title    : i18n.labelDiscoHMCCredsViosCredentialsHeading,
                                                   emptyMsg : i18n.labelDiscoHMCCredsNoCreds,
                                                   columns  : [  { id : 'name'          , label : i18n.labelNameColumn     , width : '20%' , sortable : false },
                                                                 { id : 'type'          , label : i18n.labelTypeColumn     , width : '20%' , sortable : false },
                                                                 { id : 'userName'      , label : i18n.labelUserNameColumn , width : '15%' , sortable : false },
                                                                 { id : 'actions'       , label : i18n.labelActionsColumn  , width : '15%' , sortable : false } ] } );

        /** @type {Element} */ var buttonContainerViosCredentials = ButtonFactory.buildButtonContainer();

        
        var infoParagraph = document.createElement("div");
      
        getScopesetCmd.execute();
        $(getScopesetCmd).on('executionSucceeded', $.proxy(function(event) {
            name = event.response.payload.scopeSet.displayName;
            /** @type {string} */ var displayName = name.replace("_HMCCrt_", "");

            buttonNewScope = ButtonFactory.buildButton( { type    : 'add',
                                                          grayout : true,
                                                          label   : i18n.buttonDiscoHMCScopeAdd } );
            
            $(buttonNewScope).on('click', function() {
                (new ShowCommand( { action              : 'scopeshmc/new',
                                    clearSystemMessages : true,
                                    parameters          : { pid          : pid,
                                                            scopeSetName : displayName } } ) ).execute();
            } );

            // Create Credentials Pane new button.  This is deferred so that the HMC dynamic scope set name can be determined.

            $( buttonContainerHmcCredentials )  .append( ButtonFactory.buildButtonSeparator );
            $( buttonContainerHmcCredentials )  .append(this.createAddCredentialsButton(pid, i18n.buttonDiscoHMCCredsAddHmc,   i18n.labelDiscoHMCCredsHmcType,   displayName));
            $( buttonContainerAixCredentials )  .append( ButtonFactory.buildButtonSeparator );
            $( buttonContainerAixCredentials )  .append(this.createAddCredentialsButton(pid, i18n.buttonDiscoHMCCredsAddAix,   i18n.labelDiscoHMCCredsAixType,   displayName));
            $( buttonContainerLinuxCredentials ).append( ButtonFactory.buildButtonSeparator );
            $( buttonContainerLinuxCredentials ).append(this.createAddCredentialsButton(pid, i18n.buttonDiscoHMCCredsAddLinux, i18n.labelDiscoHMCCredsLinuxType, displayName));
            $( buttonContainerViosCredentials ) .append( ButtonFactory.buildButtonSeparator );
            $( buttonContainerViosCredentials ) .append(this.createAddCredentialsButton(pid, i18n.buttonDiscoHMCCredsAddVios,  i18n.labelDiscoHMCCredsViosType,  displayName));

            // Add information to General panel
            tablePanelGeneral.appendRow( [i18n.labelNameField, displayName] );

            if (!TV.typeIsUndefined(typeof event.response.payload.scopeSet.scopeElements))  {

                var scopeElementsLength = event.response.payload.scopeSet.scopeElements.length;

                // Add row for each HMC scope
                $.each(event.response.payload.scopeSet.scopeElements, function(idx, element) {
                    var type = null;
                    if (element.type === 0) {
                        type = i18n.labelScopeSetAddress;
                    }
                    
                    /**@type function() */
                    var clickTriggerHandler = function(event) {
                        tsa.clearSysMessages();
                        var proceed = true;
                        if (!TV.typeIsUndefined(typeof event.data.confirm)) {
                            proceed = confirm(tsa.bind(tsa.i18n.labelDeleteConfirmation,  event.data.name ));
                        }
                        if (proceed) {
                            var actionCmd = new Command( { action : event.data.URL, clearSystemMessages : false });
                            actionCmd.execute();
                            $(actionCmd).on('executionSucceeded', function() {
                                (new ShowCommand( {action : 'scopesetshmc/edit',
                                                   clearSystemMessages : false,
                                                   parameters : { pid : pid }}
                                )).execute();
                            } );
                        }
                    };

                    var dActions = document.createElement("div");
                    var editIcon = IconFactory.buildActionIcon( { type    : 'edit', 
                    	                                          caption : tsa.bind(i18n.labelButtonEdit ,element.externalForm)} );
                    var runHoverText = tsa.bind(i18n.labelDiscoHMCStartDiscovery, element.externalForm);
                    var runIcon    = IconFactory.buildActionIcon( { type    : 'runb', 
                                                                    caption :  runHoverText } );
                    var deleteIcon = IconFactory.buildActionIcon( { type    : 'delete', 
                    	                                            caption : tsa.bind(i18n.labelButtonDelete ,element.externalForm) } );
                    $(editIcon).on('click', function() {
                        (new ShowCommand( {action              : 'scopeshmc/edit',
                                           clearSystemMessages : true,
                                           parameters          : {  pid          : element.pid,
                                                                    sepid        : pid,
                                                                    scopeSetName : displayName }
                                          })).execute();
                    } );

                    $(runIcon   ).on('click', { URL : "scopesetshmc/ajaxDiscoverScopeElement/" + element.pid } , clickTriggerHandler);
                    $(deleteIcon).on('click', { URL : "scopes/ajaxDelete/" + element.pid , name : element.externalForm  , confirm : true } , clickTriggerHandler);
                    var aRef = document.createElement("a");
                    $(aRef).attr('href', '#');
                    $(aRef).append(editIcon);
                    $(dActions).append(aRef);
                    $(dActions).append(tsa.NBSP);

                    if (scopeElementsLength > 1) {
                        aRef = document.createElement("a");
                        $(aRef).attr('href', '#');
                        $(aRef).append(deleteIcon);
                        $(dActions).append(aRef);
                        $(dActions).append(tsa.NBSP);
                    }

                    aRef = document.createElement("a");
                    $(aRef).attr('href', '#');
                    $(aRef).append(runIcon);
                    $(dActions).append(aRef);
                    $(dActions).append(tsa.NBSP);

                    tablePanelHmcScopes.appendRow( [type, element.externalForm, dActions ] );
                });

                // Add button for Add New HMC
                $(buttonContainerHmcScopes).append(ButtonFactory.buildButtonSeparator);
                $(buttonContainerHmcScopes).append(buttonNewScope);

                var rowButtonScopes    = tablePanelHmcScopes.tableBody.insertRow(-1);
                var tdButtonScopes     = document.createElement("td");
                var tdbuttonFillScopes = document.createElement("td");
                $(tdButtonScopes).append( buttonContainerHmcScopes );
                $(tdButtonScopes).attr("colspan", 2);
                $(tdbuttonFillScopes).attr("colspan", 4);
                $(rowButtonScopes).append(tdButtonScopes, tdbuttonFillScopes);


                var selectIpHmc = document.createElement('select');
                // Timezone selector properties...
                selectIpHmc.name = "timezone";
                selectIpHmc.id   = "timezone";

                var opt = document.createElement('option');
                opt.value = "Unassigned";

                $(opt).text("Unassigned");
                $(opt).attr('selected' , 'selected');
                $(selectIpHmc).append(opt);

                $.each(event.response.payload.scopeSet.scopeElements, function(idx, element) {
                    opt = document.createElement('option');
                    opt.value = element.pid;
                    $(opt).text(element.externalForm);
                    $(selectIpHmc).append(opt);

                });

            } else {
                // hide if there is no scopes to move
                $(buttonNewCredentials).hide();
            }

            var hmcName = name.replace("_HMCCrt_","");
            var templateInfo = function(value) {
                var pTemplate  = document.createElement("p");
                var brTemplate = $("<br class='break'>").get(0);
                $(pTemplate).append(value, brTemplate);
                return pTemplate;
            };

            var getPartitionScopeSet = new Command( { action              :  'scopesets/getScopeSets',
                                                      parameters          : { 'filter' : 'HMC'},
                                                      clearSystemMessages : false } );
            getPartitionScopeSet.execute();
            $(getPartitionScopeSet).on('executionSucceeded', function(event) {
                var scopesets = event.response.payload.scopeSets;

                if (!TV.typeIsUndefined(typeof scopesets)) {
                    var missingPartitionCreation = true;
                    $.each(scopesets, function(icaIdx, set) {
                        if (set.name.startsWith(hmcName) && set.name.match(/(_AIX|_LINUX|_WINDOWS)$/) && set.enabled){
                            $(infoParagraph).append(templateInfo(set.name));
                            missingPartitionCreation = false;
                        }
                    });

                    if (missingPartitionCreation) {
                        $(infoParagraph).append(templateInfo(i18n.labelHmcNonePartition));
                    }
                }

            });

            // Create credential panel
            // Start logic to retrieve and display credentials
            var getCredCommand = new Command( { action              : 'credentials/getCredentials',
                                                parameters          : { 'filter' : 'EVERYTHING'},
                                                clearSystemMessages : false } );
            getCredCommand.execute();
            
            $(getCredCommand).on('executionSucceeded', $.proxy(function(event) {
                // Here we use the new function
                var credentials = event.response.payload.credentials;

                $.proxy(this.retrieveCredentials("HMC",   tablePanelHmcCred,   buttonContainerHmcCredentials,   credentials, pid, name), this);
                $.proxy(this.retrieveCredentials("AIX",   tablePanelAixCred,   buttonContainerAixCredentials,   credentials, pid, name), this);
                $.proxy(this.retrieveCredentials("LINUX", tablePanelLinuxCred, buttonContainerLinuxCredentials, credentials, pid, name), this);
                $.proxy(this.retrieveCredentials("VIOS",  tablePanelViosCred,  buttonContainerViosCredentials,  credentials, pid, name), this);
            }
            , this ) );

        }, this ) );

       
        contentArea.append(tablePanelGeneral  .getDomNode());
        contentArea.append(tablePanelHmcScopes.getDomNode());
        contentArea.append(tablePanelHmcCred  .getDomNode());
        contentArea.append(tablePanelAixCred  .getDomNode());
        contentArea.append(tablePanelLinuxCred.getDomNode());
        contentArea.append(tablePanelViosCred .getDomNode());
        this.show(contentArea.getDomNode());
    };


    /*
     * Set-up add credential button on show / edit dynamic scope set panel.
     *
     * @param pid ID of the dynamic scope set.
     * @param buttonLabel Label to be associated with the button
     * @param buttonType Type of add button (HMC, AIX, Linux, VIOS)
     * @param displayName Displayable version of the dynamic scope set name
     *
     * @return Constructed button.
     */
    tsa.cmd.handlers.ScopesetshmcHandler.prototype.createAddCredentialsButton = function( pid, buttonLabel, buttonType, displayName ) {
        var addButton = ButtonFactory.buildButton( { type    : 'add',
                                                     grayout : true,
                                                     label   : buttonLabel } );

        $(addButton).on('click', function () {
             (new ShowCommand( { action              : 'credentialshmc/new',
                                 clearSystemMessages : true,
                                 parameters          : { pid          : pid,
                                                         type         : buttonType,
                                                         scopeSetName : displayName }} )
             ).execute();
        } );

        return addButton;
    };

    
    //
    // Retrieve credentials
    //
    // @param credentialType Either HMC, AIX, Linux, or VIOS
    // @param tablePanelCred Panel used for credentials.
    // @param buttonContainer Container for buttons associated with credentials.
    // @param credentials List of all credentials.
    // @param pid PID for this HMC Dynamic Scope Set.
    // @param scopesetName Name of the HMC Dynamic Scope Set.
    //
    tsa.cmd.handlers.ScopesetshmcHandler.prototype.retrieveCredentials = function(credentialType,
    		                                                                      tablePanelCred, 
    		                                                                      buttonContainer, 
    		                                                                      credentials, 
    		                                                                      pid, 
    		                                                                      scopesetName) {
        var i18n = tsa.i18n;
        
        var tablePanel      = null;
        var tablePanelProps = { title : i18n.labelDiscoCredsCredentialsHeading };
        
        var displayScopeSetName = scopesetName.replace("_HMCCrt_", "");
        
        var scopeSetNameTarget = null;
        if (credentialType === "HMC") {
        	scopeSetNameTarget = scopesetName;  // This hidden credential type is not included in credential name.
        } else {
        	scopeSetNameTarget = scopesetName.replace("_HMCCrt_", "_" + credentialType + "_HMCCrt_");
        }
        
        if (TV.typeIsObject(typeof credentials) && TV.isInstanceOf(credentials, Array) && (credentials.length > 0)) {
            var columnData = [];
            var prunedCred = [];

            // Prune credential not assigned to current hmc / aix / linux / vios scopeset
            $.each(credentials, function(credIndex, cred) {
                if (cred.name.endsWith("_HMCCrt_") && (cred.scopeSetName === scopeSetNameTarget)){
                    prunedCred.push(cred);
                }
            } );
 
            credentials = prunedCred;

            columnData.push( { id : 'name'     , label : i18n.labelDiscoCredsName     , width : '20%' , sortable : false },
                             { id : 'type'     , label : i18n.labelDiscoCredsType     , width : '20%' , sortable : false },
                             { id : 'userName' , label : i18n.labelDiscoCredsUserName , width : '15%' , sortable : false },
                             { id : 'actions'  , label : i18n.labelActionsColumn      , width : '15%' , sortable : false } ); 

            tablePanelProps.columns = columnData;
       
        } else {
            tablePanelProps.emptyMsg = i18n.labelDiscoCredsNoCreds;
        }

        tablePanel = new TablePanel(tablePanelProps);

        var credentialsLength = credentials.length;
        
        $.each(credentials, $.proxy(function(credIndex, cred) {
            // If scopeset is null or does not have the tag in the name then continue
            if (!cred.name.endsWith("_HMCCrt_")) {
                return;
            }
            cred.name = cred.name.substring(0,cred.name.indexOf("_SCOPESET_"));

            var elements    = [];
            var nameSpan    = document.createElement( 'span' );
            var nameAnchor  = document.createElement( 'a' );
            var actionsSpan = document.createElement( 'span' );

            //
            // Column 1 - Name
            //
            nameAnchor.href = '#';
            $(nameAnchor).addClass('ibm-feature-link');
            $(nameAnchor).css( { 'font-weight' : 'bold' } );
            $(nameAnchor).text(cred.name);
            $(nameAnchor).on('click', function(event) {
                ( new ShowCommand( { action     : 'credentialshmc/show/' + cred.pid, 
                                     parameters : { pid : pid }
                                   } ) ).execute();

                try {
                    event.stopPropagation();
                } catch ( error ) { /* Ignore */ }

                return false;
            } );

            $(nameSpan).append(nameAnchor);
            $(nameSpan).append(document.createTextNode(tsa.isDebugModeOn() ? ' (' + cred.properties.order + ')' : '' ));
            elements.push(nameSpan);

            //
            // Column 2 - authentication type
            //
            elements.push(this.i18nAuthType(cred.properties.authtype));
          
            //
            // Column 3 - User name:
            //
            elements.push((cred.userName !== null ) ? cred.userName : '');

            //
            // Column 4 - Actions
            //
            var editIcon     = IconFactory.buildActionIcon( { type : 'edit'     , caption : tsa.bind(i18n.labelButtonEdit,cred.name)} );
            var deleteIcon   = IconFactory.buildActionIcon( { type : 'delete'   , caption : tsa.bind(i18n.labelButtonDelete,cred.name ) } );
            var moveDownIcon = IconFactory.buildActionIcon( { type : 'movedown' , caption : tsa.bind(i18n.labelDiscoCredsMoveDown ,cred.name) } );
            var moveUpIcon   = IconFactory.buildActionIcon( { type : 'moveup'   , caption : tsa.bind(i18n.labelDiscoCredsMoveUp ,cred.name) } );

            var isFirst = (credIndex === 0);
            var isLast  = (credIndex === (credentials.length - 1));

            $(actionsSpan).append(editIcon);
            $(editIcon).on('click', function(event) { tsa.cmd.handlers.ScopesetshmcHandler.prototype.editCredentialClicked(cred.pid, pid); } );
            $(editIcon).css( { 'cursor' : 'pointer', 'margin-right' : '0.5em' } );
          
            if ((credentialsLength > 1) || (credentialType !== "HMC")) {
        	    /*
        	     * Do not allow deletion if only one credential and credential type is HMC.
          	     */
                $(actionsSpan).append(deleteIcon);
                $(deleteIcon).css( { 'cursor' : 'pointer', 'margin-right' : '0.5em' } );
                $(deleteIcon).on('click', function() { tsa.cmd.handlers.ScopesetshmcHandler.prototype.deleteCredentialClicked(cred.name, cred.pid , pid); } );
            }
          
            if (!isLast) {
                $(actionsSpan).append(moveDownIcon);
                $(moveDownIcon).on('click', function() { tsa.cmd.handlers.ScopesetshmcHandler.moveCredentialClicked(cred.pid, pid, 1); } );
            }

            if (!isFirst) {
                $(actionsSpan).append(moveUpIcon);
                $(moveUpIcon).on('click', function() { tsa.cmd.handlers.ScopesetshmcHandler.moveCredentialClicked(cred.pid, pid, -1); } );
            }

            elements.push(actionsSpan);

            // Append the row...
            tablePanelCred.appendRow(elements);

        }, this ) );

        var rowButton    = tablePanelCred.tableBody.insertRow(-1);
        var tdButton     = document.createElement("td");
        var tdbuttonFill = document.createElement("td");
        $(tdButton).append( buttonContainer );
        $(tdButton).attr("colspan", 2);
        $(tdbuttonFill).attr("colspan", 4);
        $(rowButton).append(tdButton,tdbuttonFill);
    };
    
    
    //-------------------------------------------------------------------------

    /*==============================================================================*/
    /*==============================================================================*/
    /*===                                                                        ===*/
    /*=== EVENT HANDLERS                                                         ===*/
    /*===                                                                        ===*/
    /*==============================================================================*/
    /*=============================================================================**/

    /**************************************************************************
     * DELETE CREDENTIAL EVENT HANDLER                                        *
     *                                                                        *
     * @param credentialName Name of the credential.                          *
     * @param credentialPID The credential PID.                               *
     * @param pid The PID of this HMC Dynamic Scope Set.                      *
     **************************************************************************/

    tsa.cmd.handlers.ScopesetshmcHandler.prototype.deleteCredentialClicked = function(credentialName, credentialPID , pid) {
        if (TV.typeIsString(typeof credentialPID) && (credentialPID.length > 0)) {
            if (window.confirm(tsa.bind(tsa.i18n.labelDeleteConfirmation,  credentialName ))) {
                var deleteCmd = new Command( { action     : 'credentialshmc/deleteCredential',
                                               parameters : { 'pid' : credentialPID } } );

                $(deleteCmd).on('executionSucceeded', function(event) {
                    if (!TV.typeIsUndefined(typeof event.response.sysMessages[0])) {
                        // Clear default message
                        tsa.clearSysMessages()
                        tsa.printSysMessage( { level   : 'info',
                                               message : event.response.sysMessages[0].message } );
                    }
                    
                    ( new ShowCommand( { action              : 'scopesetshmc/show',
                                         parameters          : { pid : pid },
                                         clearSystemMessages : false } ) ).execute();
                } );

                deleteCmd.execute();
            }
      
        } else {
            logger.warn( 'Invalid credential PID. Deletion request did not proceed.' );
        }
    };


    //-------------------------------------------------------------------------

    /**************************************************************************
     * EDIT CREDENTIAL EVENT HANDLER                                          *
     *                                                                        *
     * @param credentialPID The credential PID.                               *
     * @param pid The PID of this HMC Dynamic Scope Set.                      *
     **************************************************************************/

    tsa.cmd.handlers.ScopesetshmcHandler.prototype.editCredentialClicked = function( credentialPID , pid )
    {
        ( new ShowCommand( { action     : 'credentialshmc/edit/' + credentialPID,
                             parameters : { pid : pid } } ) ).execute();
    };


    //-------------------------------------------------------------------------

    /**************************************************************************
     * MOVE CREDENTIAL EVENT HANDLER                                          *
     *                                                                        *
     * @param credentialPID The credential PID.                               *
     * @param pid The PID of this HMC Dynamic Scope Set.                      *
     * @param offset Negative to move credential up, positive to move down.   *
     **************************************************************************/

    tsa.cmd.handlers.ScopesetshmcHandler.moveCredentialClicked = function( credentialPID, pid, offset ) {
        if (TV.typeIsString(typeof credentialPID) && (credentialPID.length > 0)) {
            if (TV.typeIsNumber(typeof offset) && !isNaN(offset) && (offset !== 0)) {
                var moveCmd = new Command( { action     : 'scopesetshmc/ajaxMoveCredential',
                                             parameters : { 'pid'    : credentialPID,
                                             'offset'   : offset } } );

                $(moveCmd).on('executionSucceeded', function() { ( new ShowCommand( { action              : 'scopesetshmc/show',
                                                                                      clearSystemMessages : false,
                                                                                      parameters          : { pid : pid } } ) ).execute();
                                                                   } );

                moveCmd.execute();

            } else {
                logger.error('Invalid offset.  Movement request did not proceed.');
            }

        } else {
            logger.error('Invalid credential PID.  Movement request did not proceed.');
        }
    };
    


    //-------------------------------------------------------------------------

    /*==============================================================================*/
    /*==============================================================================*/
    /*===                                                                        ===*/
    /*=== GENERAL FUNCTIONS                                                      ===*/
    /*===                                                                        ===*/
    /*==============================================================================*/
    /*=============================================================================**/


    /**************************************************************
     * CONVERT AUTHENTICATION TYPE TO DISPLAYABLE VALUE           *
     *                                                            *
     * @param authtype Authentication type string.                *
     * @return Displayable value for authentication type.         *
     **************************************************************/

    tsa.cmd.handlers.ScopesetshmcHandler.prototype.i18nAuthType = function(authtype) {
        /** @type {string} */ var i18nLabel = '';

        if (!TV.typeIsString(typeof authtype)) {
            authtype = '' + authtype;
        }

        switch(authtype) {
            case 'authType_password' : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePassword  ; break;
            case 'authType_pki'      : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePkiScoped ; break;
            default                  : i18nLabel = authtype;
        }

        return i18nLabel;
    };


        
    //  -------------------------------------------------------------------------
    return tsa.cmd.handlers.ScopesetshmcHandler;
  
    }
);
